
#include <iostream>
#include <string>

#include "GWt_abstractElement.h"
#include "GWt_elementLigneFaisceau.h"
#include "GWt_rfgun.h"
#include "GWt_drift.h"
#include "GWt_cell.h"
#include "GWt_bend.h"
#include "GWt_soleno.h"
#include "GWt_beam.h"
#include "GWt_fit.h"
#include "GWt_draggableImage.h"
#include "GWt_ligneFaisceau.h"

#include "mixedTools.h"
#include "nomdElements.h"

#include <Wt/WText>
#include <Wt/WLineEdit>
#include <Wt/WBreak>
#include <Wt/WApplication>

GWt_abstractElement::GWt_abstractElement(abstractElement* elem)
: WContainerWidget()
{
    abstractElement_ = elem;
    setStyleClass("beamLineIcon");

}


void GWt_abstractElement::initialize()
{
    // make image
    image_ = new WImage(getBigImageURL(),this);
    
    // activate slots
    image_->doubleClicked().connect(this,&GWt_abstractElement::doubleClicked);
    image_->clicked().connect(this,&GWt_abstractElement::clicked);
    addWidget(image_);
}


string* GWt_abstractElement::beginDialog()
{
    string* param = abstractElement_-> getParametersString();
    
    if ( param == NULL )
    {
        if ( static_cast<GWt_console*> (wApp->findWidget ("console"))) {
            GWt_console* console = static_cast<GWt_console*> (wApp->findWidget ("console"));
            console->addConsoleMessage(" GWt_abstractElement::beginDialog : empty parameter set for element " + abstractElement_->getNomdElement().getElementName());
        }
    }
    //   int compteur = -1;
    
    //  int nbparam = atoi(param[++compteur].c_str());
    int nbparam = atoi(param->c_str());
    if ( nbparam != abstractElement_->getNbParams())
    {
        if ( static_cast<GWt_console*> (wApp->findWidget ("console"))) {
            GWt_console* console = static_cast<GWt_console*> (wApp->findWidget ("console"));
            console->addConsoleMessage(" element seems not to be a " + abstractElement_->getNomdElement().getElementName());
        }
    }
    
    dialog_ = new WDialog("element " + abstractElement_->getNomdElement().getElementName());
    new WText(" label of the element : ",dialog_->contents());
    //  labelEdit_ = new WLineEdit(param[++compteur].c_str(), dialog_->contents());
    
    labelEdit_ = new WLineEdit( (++param)->c_str(), dialog_->contents());
    new WBreak(dialog_->contents());
    return ++param;
}

void GWt_abstractElement::updateLabelWidget()
{
   elementLigneFaiseauContainer_->setElementName(abstractElement_->getLabel());
    
    /*  string str = abstractElement_->getLabel();
     wc_item_->setText(str);
     */
}

void GWt_abstractElement::setLabelWidget()
{
    /*
     string str = abstractElement_->getLabel();
     wc_item_ = new WText(str);
     wc_item_->setObjectName("aText");
     wc_->addWidget(wc_item_);
     */
}

void GWt_abstractElement::clicked()
{
    dialog_->show();
}


void GWt_abstractElement::doubleClicked()
{
    GWt_dialog warningDialog("PSPA : Delete element",WString("Delete '")+WString(getElement()->getLabel())+"' ?", GWt_dialog::Warning,true,true);
    warningDialog.exec();

    // get LigneFaiseau widget and add new elementLigneFaiseau
    GWt_LigneFaisceau* ligneFaisceau = NULL;
    if ( static_cast<GWt_LigneFaisceau*> (wApp->findWidget ("ligneFaisceau"))) {
        ligneFaisceau = static_cast<GWt_LigneFaisceau*> (wApp->findWidget ("ligneFaisceau"));
    } else {
        return;
    }
    
    ligneFaisceau->removeElement(this);
}


WContainerWidget* GWt_abstractElement::getWidget() {
    return wc_;
}

abstractElement* GWt_abstractElement::getElement() {
    return abstractElement_;
}

GWt_abstractElement* GWt_abstractElement::ajoute(abstractElement* ptr)
{
    /*
     typedElement eType = ptr->getNomdElement().getElementType();
     
     switch (  eType ) {
     case RFgun : return new GWt_rfgun(ptr);
     case drift : return new GWt_drift(ptr);
     case cell  : return new GWt_cell(ptr);
     case bend  : return new GWt_bend(ptr);
     case soleno  : return new GWt_soleno(ptr);
     case beam  : return new GWt_beam(ptr);
     case fit  : return new GWt_fit(ptr);
     case inconnu : return NULL;
     default : return NULL;
     }
     */
    return NULL;
}


void GWt_abstractElement::createDragImage(WContainerWidget* w){

    GWt_draggableImage *result = new GWt_draggableImage(getBigImageURL(),w);

    /*
     * Set the image to be draggable, showing the other image (dragImage)
     * to be used as the widget that is visually dragged.
     */
    result->setDraggable(getMimeType(),new WImage(getSmallImageURL(),w),true);
}

WImage* GWt_abstractElement::getImage() {
    return image_;
}
